/*
 * Copyright (C) 2011 NAlek.
 * Web: http://www.nalek.org
 * E-Mail: nalek@nalek.org
 *
 * ICMPHelper version 1.1
 * Date: 12.01.2011
 *
 */
#pragma once

//
// Query Performance Timer
//
class CQPTimer
{
public:
	//    
	__int64 Get();
	//   
	double Elapsed( __int64 i64Begin, __int64 i64End );
};




//
// ICMP
//

//   
// ..     
#ifdef _MSC_VER
// The following two structures need to be packed tightly, but unlike
// Borland C++, Microsoft C++ does not do this by default.
#pragma pack(1)
#endif

//  IP  4- 
struct IPv4Header
{
	unsigned char	ucHLen:4;		// Length of the header in dwords
	unsigned char	ucVersion:4;	// Version of IP
	unsigned char	ucTOS;			// Type of service
	unsigned short	usTotalLen;		// Length of the packet in dwords
	unsigned short	usIdent;		// unique identifier
	unsigned short	usFlags;		// Flags
	unsigned char	ucTTL;			// Time to live
	unsigned char	ucProto;		// Protocol number (TCP, UDP etc)
	unsigned short	usChecksum;		// IP checksum
	unsigned long	ulSrcIP;
	unsigned long	ulDstIP;
};

//  ICMP 
struct ICMPHeader
{
	unsigned char	ucType;		// ICMP packet type
	unsigned char	ucCode;		// Type sub code
	unsigned short	usChecksum;	// ICMP Checksum
	unsigned short	usID;
	unsigned short	usSeq;
	//    ICMP ,  
	unsigned long	ulIDEx;			// 4     
	__int64			i64Timestamp;	// 8   
};

#ifdef _MSC_VER
#pragma pack()
#endif

//  ICMP 
#define		ICMP_HEADER_LEN			sizeof(ICMPHeader)
//   IP 
#define		IPv4_HEADER_LEN			sizeof(IPv4Header)
//    ICMP 
#define		MAX_ICMP_DATA_SIZE		1024
//   ICMP 
#define		MAX_ICMP_PACKET_SIZE	MAX_ICMP_DATA_SIZE+ICMP_HEADER_LEN
//    
#define		MAX_PACKET_SIZE			MAX_ICMP_DATA_SIZE+ICMP_HEADER_LEN+IPv4_HEADER_LEN+40 /* data saize + icmp header + IP header (20 bytes) + 40 (max ip header =60 bytes)*/
// ""  ICMP 
//#define		ICMPHLP_PACKET_ID			USER_ID

//
//  
//
#define		ICMPHLP_ERR_SUCCESS			0	// success
#define		ICMPHLP_ERR_WINSOCK			1	// winsock error ( WSAGetLastError() )
//#define		ICMPHLP_ERR_HOSTNOTFOUND	2	// host not found
#define		ICMPHLP_ERR_MEMALLOC		3	// memory allocation error
//#define		ICMPHLP_ERR_PACKETSIZE		4	// wrong packet size
#define		ICMPHLP_ERR_TIMEOUT			5	// receive timeout
//#define		ICMPHLP_ERR_DSTUNREACH		6	// destanation unreacheable
//#define		ICMPHLP_ERR_TTL				7	// ttl expire
#define		ICMPHLP_ERR_ICMPTYPE		8	// uncknown icmp type


//
// Definition of type and code field values.
//
#define ICMP_ECHOREPLY            0  /* echo reply */
#define ICMP_UNREACH              3  /* dest unreachable, codes: */
#define ICMP_UNREACH_NET                0  /* bad net */
#define ICMP_UNREACH_HOST               1  /* bad host */
#define ICMP_UNREACH_PROTOCOL           2  /* bad protocol */
#define ICMP_UNREACH_PORT               3  /* bad port */
#define ICMP_UNREACH_NEEDFRAG           4  /* IP_DF caused drop */
#define ICMP_UNREACH_SRCFAIL            5  /* src route failed */
#define ICMP_UNREACH_NET_UNKNOWN        6  /* unknown net */
#define ICMP_UNREACH_HOST_UNKNOWN       7  /* unknown host */
#define ICMP_UNREACH_ISOLATED           8  /* src host isolated */
#define ICMP_UNREACH_NET_PROHIB         9  /* prohibited access */
#define ICMP_UNREACH_HOST_PROHIB       10  /* ditto */
#define ICMP_UNREACH_TOSNET            11  /* bad tos for net */
#define ICMP_UNREACH_TOSHOST           12  /* bad tos for host */
#define ICMP_SOURCEQUENCH         4  /* packet lost, slow down */
#define ICMP_REDIRECT             5  /* shorter route, codes: */
#define ICMP_REDIRECT_NET               0  /* for network */
#define ICMP_REDIRECT_HOST              1  /* for host */
#define ICMP_REDIRECT_TOSNET            2  /* for tos and net */
#define ICMP_REDIRECT_TOSHOST           3  /* for tos and host */
#define ICMP_ECHO                 8  /* echo service */
#define ICMP_ROUTERADVERT         9  /* router advertisement */
#define ICMP_ROUTERSOLICIT       10  /* router solicitation */
#define ICMP_TIMXCEED            11  /* time exceeded, codes: */
#define ICMP_TIMXCEED_INTRANS           0  /* ttl==0 in transit */
#define ICMP_TIMXCEED_REASS             1  /* ttl==0 in reass */
#define ICMP_PARAMPROB           12  /* ip header bad */
#define ICMP_PARAMPROB_OPTABSENT        1  /* req. opt. absent */
#define ICMP_TSTAMP              13  /* timestamp request */
#define ICMP_TSTAMPREPLY         14  /* timestamp reply */
#define ICMP_IREQ                15  /* information request */
#define ICMP_IREQREPLY           16  /* information reply */
#define ICMP_MASKREQ             17  /* address mask request */
#define ICMP_MASKREPLY           18  /* address mask reply */



//
// ICMP helper
//
class CICMPHelper
{
private:
	//  
	CQPTimer cTimer;
	SOCKET sSock;

	int _iRecvTimeout;

	//    
	unsigned short IPChecksum( unsigned short *usBuffer, int iSize );

public:


	//  
	sockaddr_in saiDstHost;
	//    
	int iICMPDataSize;
	//   ICMP 
	int iICMPPacketSize;

#ifdef ICMPHLP_WSINIT
	//
	//  WinSock  
	//
	bool InitWinSock( WORD wVersion =MAKEWORD(2, 2) );
	bool CloseWinSock();
#endif

	//   
	bool OpenHost( const char *pHostName );
	//   
	bool OpenICMPSession( int iRecvTimeout, int iSendTimeout, int iBufferSize =12, bool bBind =false );
	//  
	bool CloseICMPSession();
	//  ICMP 
	int SendEchoRequest( unsigned short usPacketID, unsigned short usSeq, int &iSendBytes  );
	//  ICMP 
	int ReceiveEchoReply( unsigned short usPacketID, unsigned short usSeq, sockaddr_in &saiSrcHost, int &iRecvBytes, double &dRecvTime, int &iRecvTTL );
};